/*
 Written by
  Kajtar Zsolt <soci@c64.rulez.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <string.h>
#include "shorten.h"

static size_t delete_spaces(const c64c_t *s, c64c_t *out) {
    size_t cnt = 0, o = 0, i, il = strlen((const char *)s);
    size_t ac = 0, a;
    int pass, hi = 0;

    for (pass = 0; pass < 2; pass++) {
        if (pass) {
            a = il - 16;
            if (a > cnt) a = cnt;
            ac = cnt / 2;
        }
        for (i = 0; i < il; i++) {
            char ch = s[i];
            if (pass) {
                if (hi) {
                    hi = 0;
                    if (ch >= 'a' && ch <= 'z') {
                        out[o++] = ch - 32;
                        continue;
                    }
                }
                out[o++] = ch;
            }
            if (!((ch | 0x20) >= 'a' && (ch | 0x20) <= 'z') && (((unsigned char)ch) ^ 0x30) >= 10) {
                if (!pass) {
                    cnt++;
                } else {
                    ac += a;
                    if (ac >= cnt) {
                        ac -= cnt;
                        o--; hi = 1;
                    }
                }
            }
        }
    }
    out[o] = 0;
    return o;
}

static size_t delete_vowels(const c64c_t *s, c64c_t *out) {
    size_t cnt = 0, o = 0, i, il = strlen((const char *)s);
    size_t ac, a;
    int pass;

    for (pass = 0; pass < 2; pass++) {
        if (pass) {
            a = il - 16;
            if (a > cnt) a = cnt;
            ac = cnt / 2;
        }
        for (i = 0; i < il; i++) {
            char ch = s[i];
            if (pass) out[o++] = ch;
            if (i && (ch == 'a' || ch == 'e' || ch == 'i' || ch == 'o' || ch == 'u')) {
                if (!pass) {
                    cnt++;
                } else {
                    ac += a;
                    if (ac >= cnt) {
                        ac -= cnt;
                        o--;
                    }
                }
            }
        }
    }
    out[o] = 0;
    return o;
}

static size_t delete_vowels2(const c64c_t *s, c64c_t *out) {
    size_t cnt = 0, o = 0, i, il = strlen((const char *)s);
    size_t ac, a;
    int pass;

    for (pass = 0; pass < 2; pass++) {
        if (pass) {
            a = il - 16;
            if (a > cnt) a = cnt;
            ac = cnt / 2;
        }
        for (i = 0; i < il; i++) {
            char ch = s[i];
            if (pass) out[o++] = ch;
            if (i && (ch == 'A' || ch == 'E' || ch == 'I' || ch == 'O' || ch == 'U')) {
                if (!pass) {
                    cnt++;
                } else {
                    ac += a;
                    if (ac >= cnt) {
                        ac -= cnt;
                        o--;
                    }
                }
            }
        }
    }
    out[o] = 0;
    return o;
}

static size_t delete_duplicate(const c64c_t *s, c64c_t *out) {
    size_t cnt = 0, o = 0, i, il = strlen((const char *)s);
    size_t ac, a;
    int pass;

    for (pass = 0; pass < 2; pass++) {
        if (pass) {
            a = il - 16;
            if (a > cnt) a = cnt;
            ac = cnt / 2;
        }
        for (i = 0; i < il; i++) {
            char ch = s[i];
            if (pass) out[o++] = ch;
            if (ch == s[i+1]) {
                if (!pass) {
                    cnt++;
                } else {
                    ac += a;
                    if (ac >= cnt) {
                        ac -= cnt;
                        o--;
                    }
                }
            }
        }
    }
    out[o] = 0;
    return o;
}

static size_t delete_any(const c64c_t *s, c64c_t *out) {
    size_t o = 0, i, il = strlen((const char *)s);
    size_t a = il - 16;
    size_t ac = il / 2;

    for (i = 0; i < il; i++) {
        out[o++] = s[i];
        ac += a;
        if (ac >= il) {
            ac -= il;
            o--;
        }
    }
    out[o] = 0;
    return o;
}

c64c_t *shorten(const c64c_t *s, size_t l) {
    c64c_t *tmp = (c64c_t *)malloc(sizeof(c64c_t) * (l + 1));
    if (!tmp) return tmp;
    strncpy((char *)tmp, (const char *)s, l);
    tmp[l] = 0;

    l = delete_spaces(tmp, tmp);
    if (l <= 16) return tmp;
    l = delete_vowels(tmp, tmp);
    if (l <= 16) return tmp;
    l = delete_vowels2(tmp, tmp);
    if (l <= 16) return tmp;
    l = delete_duplicate(tmp, tmp);
    if (l <= 16) return tmp;
    delete_any(tmp, tmp);
    return tmp;
}
